package services

import (
	"context"
	"eta/smm_watch/utils"
	"fmt"
	"os"
	"path/filepath"
	"strings"
	"time"
)

// RefreshExcel 刷新excel文件
func RefreshExcel(ctx context.Context) (err error) {
	//utils.FileLog.Info("开始刷新了")
	fmt.Println(time.Now().Format(utils.FormatDateTime), ",开始刷新数据了")
	weekDay := time.Now().Weekday()
	// 周末不刷新数据
	if weekDay == 0 || weekDay == 6 {
		return
	}

	// 如果没有配置就不监听了
	if utils.IndexSaveDir == `` {
		return
	}

	fileList, err := GetExcelFilePath(utils.IndexSaveDir)
	if err != nil {
		fmt.Println("获取excel文件失败，ERR：", err)
		utils.FileLog.Info("开始刷新;获取excel文件失败，ERR：", err)
		return
	}
	for _, v := range fileList {
		fileExt := filepath.Ext(v)
		if fileExt == ".xlsx" || fileExt == ".xls" {
			fmt.Println(v)
			if strings.Contains(v, " ") {
				newPath := strings.Replace(v, " ", "", -1)
				tmpErr := os.Rename(v, newPath)
				fmt.Println("將路径中的空格移除")
				if tmpErr == nil {
					v = newPath
				}
			}
			AddIndexRefreshToLpush(v)
		}
	}
	return
}

// GetExcelFilePath 获取需要刷新的excel路径
func GetExcelFilePath(dirPath string) (fileList []string, err error) {
	fileList = make([]string, 0)
	list, err := os.ReadDir(dirPath)
	if err != nil {
		return
	}
	for _, v := range list {
		//tmpFilePath := dirPath + `\` + v.Name()
		// 判断字符串的最后一个字符是否为反斜杠
		if !strings.HasSuffix(dirPath, "\\") {
			// 如果不是，则在末尾添加反斜杠
			dirPath += "\\"
		}
		tmpFilePath := dirPath + v.Name()
		// 如果是目录的话，那么就查询子目录，然后返回
		if v.IsDir() {
			tmpFileList, tmpErr := GetExcelFilePath(tmpFilePath)
			if tmpErr != nil {
				return
			}
			fileList = append(fileList, tmpFileList...)
			continue
		}
		fileName := v.Name()
		////临时文件过滤
		if strings.Contains(fileName, "tmp") ||
			strings.Contains(fileName, ".TMP") ||
			strings.Contains(fileName, "~") {
			continue
		}

		fileExt := filepath.Ext(tmpFilePath)
		if fileExt == ".xlsx" || fileExt == ".xls" {
			fileList = append(fileList, tmpFilePath)
		}
	}

	return
}

// CheckTodayExcelRefresh 刷新今日未更新文件
func CheckTodayExcelRefresh(ctx context.Context) (err error) {
	utils.FileLog.Info("执行CheckExcelRefresh")

	defer func() {
		if err != nil {
			utils.FileLog.Info("CheckExcelRefresh err: %s", err.Error())
		}
	}()

	// 周末不刷新数据
	weekDay := time.Now().Weekday()
	if weekDay == 0 || weekDay == 6 {
		return
	}
	if utils.IndexSaveDir == `` {
		return
	}

	fileList, e := GetUnRefreshExcelFilePath(utils.IndexSaveDir)
	if e != nil {
		err = fmt.Errorf("get file list err: %s", e.Error())
		return
	}
	for _, v := range fileList {
		AddIndexRefreshToLpush(v)
	}

	utils.FileLog.Info("结束CheckExcelRefresh")
	return
}

// GetUnRefreshExcelFilePath 获取今天未及时刷新的excel路径
func GetUnRefreshExcelFilePath(dirPath string) (fileList []string, err error) {
	fileList = make([]string, 0)
	list, e := os.ReadDir(dirPath)
	if e != nil {
		err = fmt.Errorf("read dir err: %s", e.Error())
		return
	}

	today := time.Now().Format(utils.FormatDate) + " 00:00:00"
	todayBegin, _ := time.ParseInLocation(utils.FormatDateTime, today, time.Local)
	for _, v := range list {
		filePath := dirPath + v.Name()

		// 如果是目录的话，那么就查询子目录，然后返回
		if v.IsDir() {
			fs, e := GetUnRefreshExcelFilePath(filePath)
			if e != nil {
				err = fmt.Errorf("get dir files err: %s", e.Error())
				return
			}
			fileList = append(fileList, fs...)
			continue
		}

		fileName := v.Name()
		// 文件过滤
		if strings.Contains(fileName, "tmp") || strings.Contains(fileName, ".TMP") || strings.Contains(fileName, "~") {
			continue
		}
		fileExt := filepath.Ext(filePath)
		if fileExt != ".xlsx" && fileExt != ".xls" {
			continue
		}

		// 获取文件更新时间, 小于今日开始时间, 则进行更新
		t := GetFileModTime(filePath)
		if !t.Before(todayBegin) {
			continue
		}
		fileList = append(fileList, filePath)
	}
	return
}

// GetFileModTime 获取文件最后修改时间
func GetFileModTime(path string) time.Time {
	var err error
	defer func() {
		if err != nil {
			utils.FileLog.Info("GetFileModTime path: %s, err: %s", path, err.Error())
		}
	}()

	f, e := os.Open(path)
	if e != nil {
		err = fmt.Errorf("open file err: %s", e.Error())
		return time.Now().Local()
	}
	defer func() {
		e = f.Close()
		if e != nil {
			err = fmt.Errorf("file close err: %s", e.Error())
		}
	}()

	fi, e := f.Stat()
	if e != nil {
		err = fmt.Errorf("stat file err: %s", e.Error())
		return time.Now().Local()
	}
	return fi.ModTime()
}
