package utils

import (
	"fmt"
	beego "github.com/beego/beego/v2/adapter"
	"github.com/patrickmn/go-cache"
	"io/fs"
	"os"
	"path/filepath"
	"syscall"
	"time"
)

var (
	RunMode             string //运行模式
	REDIS_CACHE         string //缓存地址
	EDB_LIB_URL         string // 公共指标库地址
	APP_EDB_LIB_NAME_EN string
	EDB_LIB_Md5_KEY     string
	IndexSaveDir        string // 文件存放的路径
	IndexMergeSaveDir   string // 合并后的文件存放路径
	TerminalCode        string // 终端编码
	RefreshUrl          string // 刷新URL
	IsBakExcel          string // 是否备份excel
	BakIndexSaveDir     string // 备份文件存放的路径
	CacheClient         *cache.Cache
)

func init() {
	RunMode = beego.AppConfig.String("run_mode")
	config, err := beego.AppConfig.GetSection(RunMode)
	if err != nil {
		panic("配置文件读取错误 " + err.Error())
	}
	beego.Info(RunMode + "模式")

	EDB_LIB_URL = config["edb_lib_url"]
	APP_EDB_LIB_NAME_EN = config["app_edb_lib_name_en"]
	EDB_LIB_Md5_KEY = config["edb_lib_md5_key"]
	IndexSaveDir = config["index_save_dir"]
	IndexMergeSaveDir = config["index_merge_save_dir"]
	TerminalCode = config["terminal_code"]
	RefreshUrl = config["refresh_url"]
	IsBakExcel = config["is_bak_excel"]            // 是否备份excel
	BakIndexSaveDir = config["bak_index_save_dir"] // 备份文件存放的路径

	// 没有配置的话，那就是默认刷新url
	if RefreshUrl == `` {
		RefreshUrl = "http://127.0.0.1:7007/smm_chemical/refresh"
	}

	//REDIS_CACHE = config["beego_cache"]
	//Rc, Re = cache.NewCache(REDIS_CACHE) //初始化缓存

	CacheClient = cache.New(365*24*time.Hour, 365*24*time.Hour)
	ReadDefaultFileModifyTime()
}

// ReadDefaultFileModifyTime
// @Description: 监听文件变更时间
// @author: Roc
// @datetime2023-11-03 15:26:36
func ReadDefaultFileModifyTime() {
	cacheClient := CacheClient
	if cacheClient == nil {
		cacheClient = cache.New(365*24*time.Hour, 365*24*time.Hour)
	}
	err := filepath.Walk(IndexSaveDir, func(path string, info fs.FileInfo, err error) error {
		if err != nil {
			return err
		}
		if !info.IsDir() {
			fmt.Println(path)
			fileInfo, err := os.Stat(path)
			if err != nil {
				fmt.Println("os.Stat:", err.Error())
			}
			winFileAttr := fileInfo.Sys().(*syscall.Win32FileAttributeData)
			modifyTime := SecondToTime(winFileAttr.LastWriteTime.Nanoseconds() / 1e9).Format(FormatDateTime)
			fmt.Println("最后修改时间：", modifyTime)
			cacheClient.Set(path, modifyTime, 24*time.Hour)
		}
		return nil
	})
	if err != nil {
		fmt.Println("Read err:", err)
	}
}
