package watch

import (
	"encoding/json"
	"errors"
	"eta/smm_watch/utils"
	"fmt"
	"github.com/xuri/excelize/v2"
	"io/ioutil"
	"net/http"
	"path/filepath"
	"strings"
	"time"
)

// WatchIndexFileMergeRelease 监听生产的合并excel文件的处理
func WatchIndexFileMergeRelease(renameFilePath string) {
	fmt.Println("filePath:", renameFilePath)
	time.Sleep(2 * time.Second)
	if !utils.FileIsExist(renameFilePath) {
		fmt.Println("filePath is not exist:" + renameFilePath)
		return
	}

	// 备份excel
	if utils.IsBakExcel == "1" {
		copyFile(renameFilePath)
	}

	//读取文件内容
	f, err := excelize.OpenFile(renameFilePath)
	if err != nil {
		fmt.Println("OpenFile:" + renameFilePath + ",Err:" + err.Error())
		return
	}
	defer func() {
		if err := f.Close(); err != nil {
			fmt.Println("FileClose Err:" + err.Error())
			return
		}
	}()

	sheetList := f.GetSheetList()
	for _, sv := range sheetList {
		fmt.Println("sheetName:", sv)
		lenCol := 0 //指标数

		// excel表的指标数据
		indexExcelDataList := make([]map[string]string, 0)

		indexNameMap := make(map[int]string)
		indexCodeMap := make(map[int]string)
		sourceMap := make(map[int]string)
		unitMap := make(map[int]string)
		frequencyMap := make(map[int]string)

		rows, err := f.GetRows(sv)
		if err != nil {
			fmt.Println("GetRows Err:", err)
			return
		}

		for row, cols := range rows {
			lenCol = len(cols) - 1
			if lenCol < 1 { // 没有指标，不处理
				continue
			}
			// 指标名称
			if row == 0 {
				for i := 1; i <= lenCol; i++ {
					tmpIndexExcelDataList := make(map[string]string, 0)
					indexExcelDataList = append(indexExcelDataList, tmpIndexExcelDataList)
				}
			}

			firstCol := cols[0]
			if strings.Contains(firstCol, "名称") { // 指标名称
				for i := 1; i <= lenCol; i++ {
					indexNameMap[i-1] = cols[i]
				}
			} else if strings.Contains(firstCol, "指标Id") {
				for i := 1; i <= lenCol; i++ {
					indexCodeMap[i-1] = cols[i]
				}
			} else if strings.Contains(firstCol, "单位") {
				for i := 1; i <= lenCol; i++ {
					unitMap[i-1] = cols[i]
				}
			} else if strings.Contains(firstCol, "频率") { // 更新频率
				for i := 1; i <= lenCol; i++ {
					tmpFrequency := cols[i]
					//fmt.Println("3:", valText)
					if strings.Contains(tmpFrequency, "日") ||
						strings.Contains(tmpFrequency, "月") ||
						strings.Contains(tmpFrequency, "周") ||
						strings.Contains(tmpFrequency, "季") ||
						strings.Contains(tmpFrequency, "年") ||
						strings.Contains(tmpFrequency, "半年") {
						if !strings.Contains(tmpFrequency, "度") {
							tmpFrequency = tmpFrequency + "度"
						}
						frequencyMap[i-1] = tmpFrequency
					}
				}
			} else if strings.Contains(firstCol, "来源") { //数据来源
				for i := 1; i <= lenCol; i++ {
					sourceMap[i-1] = cols[i]
				}
			} else if strings.Contains(firstCol, "时间区间") {
				continue
			} else { // 指标数据
				date := ``
				for k, col := range cols {
					if k == 0 {
						date = col
						continue
					}
					if date == `` {
						continue
					}
					if col != `` {
						// 指标列与数据列不匹配，那么就过滤
						if len(indexExcelDataList) < k {
							continue
						}
						indexExcelDataList[k-1][date] = col
					}
				}
			}

		}

		for k, excelDataMap := range indexExcelDataList {
			//fmt.Println(k, renameFilePath, renameFilePath, indexNameMap[k], indexCodeMap[k], unitMap[k], frequencyMap[k], sourceMap)
			dataJson, _ := json.Marshal(excelDataMap)
			utils.FileLog.Info(indexCodeMap[k] + "\n" + string(dataJson) + "\n" + renameFilePath)
			smmIndexHandle(renameFilePath, renameFilePath, indexNameMap[k], indexCodeMap[k], unitMap[k], frequencyMap[k], sourceMap[k], excelDataMap)
		}
	}
}

// copyFile
// @Description: 复制excel，留档，避免出现问题无法追踪
// @author: Roc
// @datetime 2024-03-11 13:52:42
// @param srcPath string
// @return error
func copyFile(srcPath string) error {
	// 如果没有设置备份目录，那么就不备份
	if utils.BakIndexSaveDir == `` {
		return nil
	}

	fileName := filepath.Base(srcPath)
	//fmt.Println(fileName)
	dstFilename := fmt.Sprintf("%s_%s", time.Now().Format(utils.FormatDateTimeUnSpace), fileName)

	return utils.CopyFile(srcPath, utils.BakIndexSaveDir, dstFilename)

}

// smmIndexHandle 有色数据处理
func smmIndexHandle(baseFilePath, renameFilePath, indexName, indexCode, unit, frequency, source string, excelDataMap map[string]string) {
	// 修改指标信息
	if indexName == "" {
		utils.FileLog.Info("未刷新到指标数据:indexName为空")
		return
	}
	if indexCode == "" {
		utils.FileLog.Info("未刷新到指标数据，indexCode为空")
		return
	}
	//if unit == "" {
	//	utils.FileLog.Info("未刷新到指标数据，单位为空")
	//	return
	//}
	if unit == "" {
		utils.FileLog.Info("未刷新到指标数据，频度为空")
		return
	}

	param := make(map[string]interface{})
	param["BaseFilePath"] = baseFilePath
	param["RenameFilePath"] = renameFilePath
	param["TerminalCode"] = utils.TerminalCode
	param["IndexName"] = indexName
	param["IndexCode"] = indexCode
	param["Source"] = source
	param["Unit"] = unit
	param["Frequency"] = frequency
	param["ExcelDataMap"] = excelDataMap
	_, e := postRefreshEdbData(param)
	if e != nil {
		utils.FileLog.Info("postRefreshEdbData err: %s", e.Error())
	}
}

type BaseResponse struct {
	Ret         int
	Msg         string
	ErrMsg      string
	ErrCode     string
	Data        interface{}
	Success     bool `description:"true 执行成功，false 执行失败"`
	IsSendEmail bool `json:"-" description:"true 发送邮件，false 不发送邮件"`
	IsAddLog    bool `json:"-" description:"true 新增操作日志，false 不新增操作日志" `
}

// postRefreshEdbData 刷新指标数据
func postRefreshEdbData(param map[string]interface{}) (resp *BaseResponse, err error) {
	urlStr := "smm/handle/excel_data"

	postUrl := utils.EDB_LIB_URL + urlStr
	postData, err := json.Marshal(param)
	if err != nil {
		return
	}
	result, err := HttpPost(postUrl, string(postData), "application/json")
	if err != nil {
		return
	}
	utils.FileLog.Info(" Refresh Result: " + string(result))
	err = json.Unmarshal(result, &resp)
	if err != nil {
		return
	}
	return resp, nil
}

// RefreshConfigResponse
// @Description: 获取刷新配置列表
type RefreshConfigResponse struct {
	Ret         int
	Msg         string
	ErrMsg      string
	ErrCode     string
	Data        []EdbRefreshDefaultConfig
	Success     bool `description:"true 执行成功，false 执行失败"`
	IsSendEmail bool `json:"-" description:"true 发送邮件，false 不发送邮件"`
	IsAddLog    bool `json:"-" description:"true 新增操作日志，false 不新增操作日志" `
}

// EdbRefreshDefaultConfig
// @Description: 指标的默认刷新时间配置
type EdbRefreshDefaultConfig struct {
	Id                  int       `orm:"column(id);pk"`
	Source              int       `description:"来源"`
	SubSource           int       `description:"来源名称"`
	Frequency           string    `description:"频度"`
	RefreshFrequency    string    `description:"刷新频率"`
	RefreshFrequencyDay int       `description:"具体刷新的日期"`
	RefreshTime         string    `description:"刷新时间"`
	RefreshAllData      int       `description:"是否刷新所有数据，0：否，1：刷新所有数据"`
	RefreshDataNum      int       `description:"刷新单元格数"`
	ModifyTime          time.Time `description:"最晚一次的更新时间"`
	CreateTime          time.Time `description:"添加时间"`
}

// GetEdbRefreshConfigReq
// @Description: 获取默认配置的请求参数
type GetEdbRefreshConfigReq struct {
	Source    int `description:"指标来源ID"`
	SubSource int `description:"子来源"`
}

// GetRefreshConfigList 获取刷新配置列表
func GetRefreshConfigList() (resp *RefreshConfigResponse, err error) {
	urlStr := "/edb_refresh/config/default/list"

	if utils.EDB_LIB_URL == `` {
		err = errors.New("刷新未配置")
		return
	}
	postUrl := utils.EDB_LIB_URL + urlStr
	postData, err := json.Marshal(GetEdbRefreshConfigReq{
		Source:    utils.DATA_SOURCE_YS,
		SubSource: 0,
	})
	if err != nil {
		return
	}
	result, err := HttpPost(postUrl, string(postData), "application/json")
	if err != nil {
		return
	}
	if result != nil {
		utils.FileLog.Debug(" GetRefreshConfigList Result: " + string(result))
		err = json.Unmarshal(result, &resp)
		if err != nil {
			return
		}
		return resp, nil
	}
	return nil, err
}

func HttpPost(url, postData string, params ...string) ([]byte, error) {
	body := ioutil.NopCloser(strings.NewReader(postData))
	client := &http.Client{}
	req, err := http.NewRequest("POST", url, body)
	if err != nil {
		return nil, err
	}
	contentType := "application/x-www-form-urlencoded;charset=utf-8"
	if len(params) > 0 && params[0] != "" {
		contentType = params[0]
	}
	req.Header.Set("Content-Type", contentType)
	req.Header.Set("authorization", utils.MD5(utils.APP_EDB_LIB_NAME_EN+utils.EDB_LIB_Md5_KEY))
	resp, err := client.Do(req)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()
	b, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}
	fmt.Println("HttpPost:" + string(b))
	return b, err
}
