package watch

import (
	"context"
	"eta/smm_watch/utils"
	"fmt"
	"github.com/patrickmn/go-cache"
	"io/fs"
	"os"
	"path/filepath"
	"syscall"
	"time"
)

func ReadWatchIndexFile(ctx context.Context) (err error) {
	defer func() {
		if err != nil {
			fmt.Println("ReadWatchIndexFile Err:" + err.Error())
		}
	}()
	cacheClient := utils.CacheClient
	if cacheClient == nil {
		cacheClient = cache.New(365*24*time.Hour, 365*24*time.Hour)
	}
	err = filepath.Walk(utils.IndexSaveDir, func(path string, info fs.FileInfo, err error) error {
		if err != nil {
			return err
		}
		if !info.IsDir() {
			fileInfo, err := os.Stat(path)
			if err != nil {
				fmt.Println("os.Stat:", err.Error())
			}
			winFileAttr := fileInfo.Sys().(*syscall.Win32FileAttributeData)
			modifyTimeStr := utils.SecondToTime(winFileAttr.LastWriteTime.Nanoseconds() / 1e9).Format(utils.FormatDateTime)

			existModifyTime, ok := cacheClient.Get(path)
			if ok {
				existModifyTimeStr := existModifyTime.(string)
				if existModifyTimeStr != modifyTimeStr {
					WatchIndexFileMergeRelease(path)
				}
			} else {
				WatchIndexFileMergeRelease(path)
			}
			cacheClient.Delete(path)
			cacheClient.Set(path, modifyTimeStr, 24*time.Hour)
		}
		return nil
	})

	return
}
